--[[

	SimpleBagStats: shows simple stats about your bags

	- includes coloring, individual bags, optional cosmos integration and command line integration
	
	by phreak

	credits go to: Nevir (for his hsb_to_rgb function)

]]
	
SimpleBagStats_enabled = 0;
SimpleBagStats_individual = 0;
SimpleBagStats_colored = 0;
SimpleBagStats_Cosmos_Registered = 0;

SimpleBagStats_color_low_hue    = 0;
SimpleBagStats_color_high_hue   = 120;
SimpleBagStats_color_saturation = 100;
SimpleBagStats_color_brightness = 75;

hsb_to_rgb_table = {};
hsb_to_rgb_table[0] = {"max", "+",   "min"};
hsb_to_rgb_table[1] = {"-",   "max", "min"};
hsb_to_rgb_table[2] = {"min", "max", "+"};
hsb_to_rgb_table[3] = {"min", "-",   "max"};
hsb_to_rgb_table[4] = {"+",   "min", "max"};
hsb_to_rgb_table[5] = {"max", "min", "-"};

color_storage = {};

function SimpleBagStats_Register()
	RegisterForSave("SimpleBagStats_enabled");
	RegisterForSave("SimpleBagStats_individual");
	RegisterForSave("SimpleBagStats_colored");

	if ( Cosmos_RegisterConfiguration ) and ( Cosmos_UpdateValue ) then
		SimpleBagStats_Register_Cosmos();
	else
		SlashCmdList["SIMPLEBAGSTATSSLASHMAIN"] = SimpleBagStats_Main_ChatCommandHandler;
		SLASH_SIMPLEBAGSTATSSLASHMAIN1 = "/simplebagstats";
		SLASH_SIMPLEBAGSTATSSLASHMAIN2 = "/sbs";
		this:RegisterEvent("VARIABLES_LOADED");
	end
end

function SimpleBagStats_Register_Cosmos()
	if ( ( Cosmos_UpdateValue ) and ( Cosmos_RegisterConfiguration ) and ( SimpleBagStats_Cosmos_Registered == 0 ) ) then
		Cosmos_RegisterConfiguration(
			"COS_SIMPLEBAGSTATS",
			"SECTION",
			"SimpleBagStats",
			"SimpleBagStats Addon"
		);
		Cosmos_RegisterConfiguration(
			"COS_SIMPLEBAGSTATS_HEADER",
			"SEPARATOR",
			"SimpleBagStats",
			"SimpleBagStats Addon"
		);
		Cosmos_RegisterConfiguration(
			"COS_SIMPLEBAGSTATS_ENABLED",
			"CHECKBOX",
			SIMPLEBAGSTATS_TEXT_ENABLE,
			SIMPLEBAGSTATS_TEXT_ENABLE_INFO,
			SimpleBagStats_enable,
			1,
			1
		);
		Cosmos_RegisterConfiguration(
			"COS_SIMPLEBAGSTATS_INDIVIDUAL",
			"CHECKBOX",
			SIMPLEBAGSTATS_TEXT_INDIVIDUAL,
			SIMPLEBAGSTATS_TEXT_INDIVIDUAL_INFO,
			SimpleBagStats_individualize,
			0,
			1
		);
		Cosmos_RegisterConfiguration(
			"COS_SIMPLEBAGSTATS_COLORS",
			"CHECKBOX",
			SIMPLEBAGSTATS_TEXT_COLOR,
			SIMPLEBAGSTATS_TEXT_COLOR_INFO,
			SimpleBagStats_color,
			0,
			1
		);
		SimpleBagStats_Cosmos_Registered = 1;
	end
end

function SimpleBagStats_OnLoad()
	this:RegisterEvent("VARIABLES_LOADED");
  this:RegisterEvent("BAG_UPDATE");
	this:RegisterEvent("UNIT_MODEL_CHANGED");
	this:RegisterEvent("UNIT_INVENTORY_CHANGED");
	if( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("phreaks SimpleBagStats AddOn 1.0 loaded");
	end
	SimpleBagStats_Register();
end

function SimpleBagStats_Update()
	local SBS_totalSlots = 0;
	local SBS_totalUsedSlots = 0;
	local SBS_numSlots = 0;
	local SBS_tempUsedSlots = 0;
	local SBS_bag0_numSlots = 0;
	local SBS_bag0_usedSlots = 0;
	local SBS_bag1_numSlots = 0;
	local SBS_bag1_usedSlots = 0;
	local SBS_bag2_numSlots = 0;
	local SBS_bag2_usedSlots = 0;
	local SBS_bag3_numSlots = 0;
	local SBS_bag3_usedSlots = 0;
	local SBS_bag4_numSlots = 0;
	local SBS_bag4_usedSlots = 0;
	local SimpleBagStats_bagratio = 0;
	
	saturation = SimpleBagStats_color_saturation;
	brightness = SimpleBagStats_color_brightness;

	
	for bag = 0, 4, 1 do
		SBS_numSlots = GetContainerNumSlots(bag);
		if (SBS_numSlots ~= 0) then
			SBS_totalSlots = SBS_totalSlots + SBS_numSlots;
				for slot = 1, SBS_numSlots, 1 do
					if (GetContainerItemInfo(bag, slot)) then
						SBS_totalUsedSlots = SBS_totalUsedSlots + 1;
						SBS_tempUsedSlots = SBS_tempUsedSlots + 1;
					end
					if (bag == 0) then
						SBS_bag0_numSlots = SBS_numSlots;
						SBS_bag0_usedSlots = SBS_tempUsedSlots;
					elseif (bag == 1) then
						SBS_bag1_numSlots = SBS_numSlots;
						SBS_bag1_usedSlots = SBS_tempUsedSlots;
					elseif (bag == 2) then
						SBS_bag2_numSlots = SBS_numSlots;
						SBS_bag2_usedSlots = SBS_tempUsedSlots;
					elseif (bag == 3) then
						SBS_bag3_numSlots = SBS_numSlots;
						SBS_bag3_usedSlots = SBS_tempUsedSlots;
					elseif (bag == 4) then
						SBS_bag4_numSlots = SBS_numSlots;
						SBS_bag4_usedSlots = SBS_tempUsedSlots;
					end
				end
				SBS_tempUsedSlots = 0;
		end
	end
	
	if (SimpleBagStats_enabled == 1) then
		if (SimpleBagStats_individual == 1) then
			SimpleBagStatsBag0Text:SetText(SBS_bag0_usedSlots.."/"..SBS_bag0_numSlots);
			if (SimpleBagStats_colored == 1) then
				SimpleBagStats_bagratio = (SBS_bag0_usedSlots / SBS_bag0_numSlots);
				hue=(SimpleBagStats_color_high_hue - math.floor(SimpleBagStats_bagratio*SimpleBagStats_color_high_hue));
				SimpleBagStatsBag0Text:SetTextColor(hsb_to_rgb(hue, saturation, brightness));
			else
				SimpleBagStatsBag0Text:SetTextColor(1,1,1);
			end
			if (SBS_bag1_numSlots ~= 0) then
				SimpleBagStatsBag1Text:SetText(SBS_bag1_usedSlots.."/"..SBS_bag1_numSlots);
				if (SimpleBagStats_colored == 1) then
					SimpleBagStats_bagratio = (SBS_bag1_usedSlots / SBS_bag1_numSlots);
					hue=(SimpleBagStats_color_high_hue - math.floor(SimpleBagStats_bagratio*SimpleBagStats_color_high_hue));
					SimpleBagStatsBag1Text:SetTextColor(hsb_to_rgb(hue, saturation, brightness));
				else
					SimpleBagStatsBag1Text:SetTextColor(1,1,1);
				end
			else
				SimpleBagStatsBag1Text:SetText("");
			end
			if (SBS_bag2_numSlots ~= 0) then
				SimpleBagStatsBag2Text:SetText(SBS_bag2_usedSlots.."/"..SBS_bag2_numSlots);
				if (SimpleBagStats_colored == 1) then
					SimpleBagStats_bagratio = (SBS_bag2_usedSlots / SBS_bag2_numSlots);
					hue=(SimpleBagStats_color_high_hue - math.floor(SimpleBagStats_bagratio*SimpleBagStats_color_high_hue));
					SimpleBagStatsBag2Text:SetTextColor(hsb_to_rgb(hue, saturation, brightness));
				else
					SimpleBagStatsBag2Text:SetTextColor(1,1,1);
				end
			else
				SimpleBagStatsBag2Text:SetText("");
			end
			if (SBS_bag3_numSlots ~= 0) then
				SimpleBagStatsBag3Text:SetText(SBS_bag3_usedSlots.."/"..SBS_bag3_numSlots);
				if (SimpleBagStats_colored == 1) then
					SimpleBagStats_bagratio = (SBS_bag3_usedSlots / SBS_bag3_numSlots);
					hue=(SimpleBagStats_color_high_hue - math.floor(SimpleBagStats_bagratio*SimpleBagStats_color_high_hue));
					SimpleBagStatsBag3Text:SetTextColor(hsb_to_rgb(hue, saturation, brightness));
				else
					SimpleBagStatsBag3Text:SetTextColor(1,1,1);
				end
			else
				SimpleBagStatsBag3Text:SetText("");
			end
			if (SBS_bag4_numSlots ~= 0) then
				SimpleBagStatsBag4Text:SetText(SBS_bag4_usedSlots.."/"..SBS_bag4_numSlots);
				if (SimpleBagStats_colored == 1) then
					SimpleBagStats_bagratio = (SBS_bag4_usedSlots / SBS_bag4_numSlots);
					hue=(SimpleBagStats_color_high_hue - math.floor(SimpleBagStats_bagratio*SimpleBagStats_color_high_hue));
					SimpleBagStatsBag4Text:SetTextColor(hsb_to_rgb(hue, saturation, brightness));
				else
					SimpleBagStatsBag4Text:SetTextColor(1,1,1);
				end
			else
				SimpleBagStatsBag4Text:SetText("");
			end
		else
			SimpleBagStatsBag0Text:SetText(SBS_totalUsedSlots.."/"..SBS_totalSlots);
			if (SimpleBagStats_colored == 1) then
				SimpleBagStats_bagratio = (SBS_totalUsedSlots / SBS_totalSlots);
				hue=(SimpleBagStats_color_high_hue - math.floor(SimpleBagStats_bagratio*SimpleBagStats_color_high_hue));
				SimpleBagStatsBag0Text:SetTextColor(hsb_to_rgb(hue, saturation, brightness));
			else
				SimpleBagStatsBag0Text:SetTextColor(1,1,1);
			end
			
			SimpleBagStatsBag1Text:SetText("");
			SimpleBagStatsBag2Text:SetText("");
			SimpleBagStatsBag3Text:SetText("");
			SimpleBagStatsBag4Text:SetText("");
		end
	else
		SimpleBagStatsBag0Text:SetText("");
		SimpleBagStatsBag1Text:SetText("");
		SimpleBagStatsBag2Text:SetText("");
		SimpleBagStatsBag3Text:SetText("");
		SimpleBagStatsBag4Text:SetText("");
	end
end

function SimpleBagStats_enable(toggle)
	if (toggle == -1) then
		if (SimpleBagStats_enabled <= 0) then
			SimpleBagStats_enabled = 1;
		else
			SimpleBagStats_enabled = 0;
		end
		SimpleBagStats_Update();
	else
		SimpleBagStats_enabled = toggle;
		SimpleBagStats_Update();
	end
end

function SimpleBagStats_individualize(toggle)
	if (toggle == -1) then
		if (SimpleBagStats_individual <= 0) then
			SimpleBagStats_individual = 1;
		else
			SimpleBagStats_individual = 0;
		end
		SimpleBagStats_Update();
	else
		SimpleBagStats_individual = toggle;
		SimpleBagStats_Update();
	end
end

function SimpleBagStats_color(toggle)
	if (toggle == -1) then
		if (SimpleBagStats_colored <= 0) then
			SimpleBagStats_colored = 1;
		else
			SimpleBagStats_colored = 0;
		end
		SimpleBagStats_Update();
	else
		SimpleBagStats_colored = toggle;
		SimpleBagStats_Update();
	end
end

function SimpleBagStats_OnEvent(event)
  if ((event == "BAG_UPDATE") 
	or (event == "UNIT_MODEL_CHANGED")
	or (event == "UNIT_INVENTORY_CHANGED")
	) then
     SimpleBagStats_Update();
  end
end

function hsb_to_rgb(hue, saturation, brightness)
	local index, min, max, rgb;

	hue = math.mod(hue, 360);
	index = math.floor(hue / 60);

	min = (brightness / 100) * ((100 - saturation) / 100);
	max = (brightness / 100);

	for color = 1, 3, 1 do
		if (hsb_to_rgb_table[index][color] == "min") then
			color_storage[color] = min;
		elseif (hsb_to_rgb_table[index][color] == "max") then
			color_storage[color] = max;
		elseif (hsb_to_rgb_table[index][color] == "+") then
			color_storage[color] = min + (math.mod(hue, 60) / 60) * (max - min);
		else
			color_storage[color] = max - (math.mod(hue, 60) / 60) * (max - min);
		end
	end

	return color_storage[1], color_storage[2], color_storage[3];
end

function SimpleBagStats_Extract_NextParameter(msg)
	local params = msg;
	local command = params;
	local index = strfind(command, " ");
	if ( index ) then
		command = strsub(command, 1, index-1);
		params = strsub(params, index+1);
	else
		params = "";
	end
	return command, params;
end

function SimpleBagStats_Main_ChatCommandHandler(msg)
	if ( ( not msg) or ( strlen(msg) <= 0 ) ) then
		if( DEFAULT_CHAT_FRAME ) then
			DEFAULT_CHAT_FRAME:AddMessage(SIMPLEBAGSTATS_TEXT_COMMANDS);
		end
		return;
	end
	
	local commandName, params = SimpleBagStats_Extract_NextParameter(msg);
	
	if ( ( commandName ) and ( strlen(commandName) > 0 ) ) then
		commandName = string.lower(commandName);
	else
		commandName = "";
	end
	
	if ( strfind( commandName, "enable" ) ) then
		SimpleBagStats_enable(-1);
		return;
	elseif ( strfind( commandName, "color" ) ) then
		SimpleBagStats_color(-1);
		return;
	elseif ( strfind( commandName, "individual" ) ) then
		SimpleBagStats_individualize(-1);
		return;
	else
		if( DEFAULT_CHAT_FRAME ) then
			DEFAULT_CHAT_FRAME:AddMessage(SIMPLEBAGSTATS_TEXT_COMMANDS);
		end
		return;
	end
end